from gcs_cli.helpers import requires_login
from gcs_cli.parsing import globus_command, include_private_policies_opt
from gcs_cli.prettyprint import FORMAT_TEXT_TABLE, formatted_print
from gcs_cli.services.gcs import connector_id_to_display_name, get_client


@globus_command("list", short_help="List storage gateways")
@include_private_policies_opt
@requires_login
def storage_gateway_list(include_private_policies):
    """
    List all storage gateways on the endpoint the GCS cli is logged into
    """
    client = get_client()

    include = []
    if include_private_policies:
        include = ["private_policies"]

    res = client.paginated.get_storage_gateway_list(include)

    # output
    fields = [
        ("Display Name", "display_name"),
        ("ID", "id"),
        ("Connector", lambda x: connector_id_to_display_name(x["connector_id"])),
        ("High Assurance", "high_assurance"),
    ]
    formatted_print(res, text_format=FORMAT_TEXT_TABLE, fields=fields)
