import json


def sorted_json_field(key):
    """
    Define sorted JSON output for text output containing complex types.  While dicts are
    order preserving in py3.6+ [1], we have to support and test on py3.5 Therefore, it's
    practical for us to make these fields present in a stable way so that our tests work
    on 3.5


    [1] https://docs.python.org/3/whatsnew/3.6.html#new-dict-implementation
    """

    def field_func(data):
        return json.dumps(data[key], sort_keys=True)

    field_func._filter_key = key
    return field_func


def filter_fields(check_fields, container):
    """
    Given a set of fields, this is a list of fields actually found in some containing
    object.

    Always includes keyfunc fields unless they set the magic _filter_key attribute
    sorted_json_field above is a good example of doing this
    """
    fields = []
    for name, key in check_fields:
        check_key = key
        if callable(key) and hasattr(key, "_filter_key"):
            check_key = key._filter_key

        # if it's a string lookup, check if it's contained (and skip if not)
        if isinstance(check_key, str) and check_key not in container:
            continue

        # anything else falls through to success
        # includes keyfuncs which don't set _filter_key
        fields.append((name, key))
    return fields
