"""
Define a command decorator which requires the user be logged-in before running the
command.
"""

import functools

import click

from gcs_cli.config import GCS_CLIENT_DATA_CONFIG_KEY, token_storage_adapter


def is_logged_in():
    adapter = token_storage_adapter()
    return bool(adapter.read_config(GCS_CLIENT_DATA_CONFIG_KEY))


def requires_login(command):
    @functools.wraps(command)
    def wrapped(*args, **kwargs):
        if not is_logged_in():
            raise click.ClickException(
                "You must login first, before running this command!"
            )
        return command(*args, **kwargs)

    return wrapped
