import click


class _State:
    def __init__(self):
        self.output_format = "text"
        self.explicit_host = None
        self.show_stack_trace = False


def format_option(f):
    def callback(ctx, param, value):
        if not value:
            return
        state = ctx.ensure_object(_State)
        state.output_format = value.lower()

    return click.option(
        "-F",
        "--format",
        type=click.Choice(["text", "json"], case_sensitive=False),
        help="Output format for stdout. Defaults to text",
        expose_value=False,
        callback=callback,
    )(f)


def explicit_host_option(f):
    def callback(ctx, param, value):
        if not value:
            return
        state = ctx.ensure_object(_State)
        state.explicit_host = value

    return click.option(
        "--use-explicit-host",
        hidden=True,
        envvar="GCS_CLI_EXPLICIT_HOST",
        expose_value=False,
        callback=callback,
    )(f)


def show_stack_trace_option(f):
    def callback(ctx, param, value):
        if not value:
            return
        state = ctx.find_root().ensure_object(_State)
        state.show_stack_trace = value

    return click.option(
        "--show-stack-trace",
        hidden=True,
        envvar="GCS_CLI_SHOW_STACK_TRACE",
        is_flag=True,
        expose_value=False,
        callback=callback,
    )(f)


def get_explicit_host():
    ctx = click.get_current_context(silent=True)
    if ctx:
        state = ctx.ensure_object(_State)
        return state.explicit_host


def outformat_is_text():
    ctx = click.get_current_context()
    state = ctx.ensure_object(_State)
    return state.output_format == "text"


def outformat_is_json():
    ctx = click.get_current_context()
    state = ctx.ensure_object(_State)
    return state.output_format == "json"


def get_show_stack_trace():
    ctx = click.get_current_context()
    state = ctx.ensure_object(_State)
    return state.show_stack_trace
