from typing import List

import globus.manager.api.errors as errors
from globus.manager.constants import ConfigurationObjectTag

from . import UserCredential
from .decorators import checks_property


class BoxUserCredential(UserCredential):
    POLICY_DATA_TYPE = "box_user_credential_policies#1.0.0"
    __mapper_args__ = {
        "polymorphic_identity": (
            ConfigurationObjectTag.USER_CREDENTIAL.name
            + "7c100eae-40fe-11e9-95a3-9cb6d0d9fd63"
        )
    }
    connector_properties = []  # type: List[str]

    def __init__(self, appsync_data=None, doc=None, storage_gateway=None):
        if doc:
            username = doc["username"]
            display_name = storage_gateway.get_user_display_name(username)
            if display_name is None:
                raise errors.InvalidUserError(username)
            policies = doc.get("policies")
            if not policies:
                policies = {"DATA_TYPE": self.POLICY_DATA_TYPE}

            doc = {
                **doc,
                **{
                    "display_name": display_name,
                    "invalid": doc.get("invalid", False),
                    "deleted": False,
                    "provisioned": True,
                    "policies": policies,
                },
            }

        super().__init__(
            appsync_data=appsync_data, doc=doc, storage_gateway=storage_gateway
        )

    @checks_property(prop="display_name")
    def check_display_name(self, value: str):
        if value and getattr(self, "display_name", None) != value:
            raise AttributeError()

    def patch(self, **doc):
        display_name = doc.pop("display_name", getattr(self, "display_name", None))
        doc["display_name"] = display_name
        super().patch(**doc)

    def put(self, **doc):
        display_name = doc.pop("display_name", getattr(self, "display_name", None))
        doc["display_name"] = display_name
        super().put(**doc)

    @property
    def connector_id(self):
        return "7c100eae-40fe-11e9-95a3-9cb6d0d9fd63"

    @connector_id.setter
    def connector_id(self, value):
        pass

    def __repr__(self):
        return "<BoxUserCredential {}>".format(self.id)
