from typing import List

from globus.manager.constants import ConfigurationObjectTag
import globus.manager.api.errors as errors
from .user_credential import UserCredential


class PosixUserCredential(UserCredential):
    POLICY_DATA_TYPE = "posix_user_credential_policies#1.0.0"
    __mapper_args__ = {
        "polymorphic_identity": (
            ConfigurationObjectTag.USER_CREDENTIAL.name
            + "145812c8-decc-41f1-83cf-bb2a85a2a70b"
        )
    }
    connector_properties = []  # type: List[str]

    def __init__(self, appsync_data=None, doc=None, storage_gateway=None):
        if doc:
            username = doc["username"]
            passwd_entry, groups = storage_gateway.lookup_unix_info(username)
            if not passwd_entry:
                raise errors.InvalidUserError(username)
            display_name = passwd_entry.pw_gecos.split(",")[0] or username
            policy = doc.get("policy")
            if not policy:
                policy = {"DATA_TYPE": self.POLICY_DATA_TYPE}
            doc = {
                **doc,
                **{
                    "display_name": display_name,
                    "invalid": False,
                    "deleted": False,
                    "provisioned": True,
                    "policy": policy,
                },
            }

        super().__init__(
            appsync_data=appsync_data, doc=doc, storage_gateway=storage_gateway
        )

    def put(self, **doc):
        display_name = self.storage_gateway.get_user_display_name(self.username)
        super().put(**{**doc, **{"display_name": display_name}})

    def patch(self, **doc):
        doc.pop("display_name", None)
        super().patch(**doc)

    @property
    def connector_id(self):
        return "145812c8-decc-41f1-83cf-bb2a85a2a70b"

    @connector_id.setter
    def connector_id(self, value):
        pass

    def __repr__(self):
        return "<PosixUserCredential {}>".format(self.id)
