import marshmallow as ma
from marshmallow_oneofschema import OneOfSchema


class ExternalIdentityMappingSchema(ma.Schema):
    _DATA_TYPE = "external_identity_mapping#1.0.0"
    DATA_TYPE = ma.fields.Str(
        default=_DATA_TYPE, validate=[ma.validate.Equal(_DATA_TYPE)]
    )
    command = ma.fields.List(ma.fields.Str)


class ExpressionIdentityMappingEntrySchema(ma.Schema):
    source = ma.fields.Str()
    match = ma.fields.Str()
    ignore_case = ma.fields.Boolean()
    literal = ma.fields.Boolean()
    output = ma.fields.Str()


class ExpressionIdentityMappingSchema(ma.Schema):
    _DATA_TYPE = "expression_identity_mapping#1.0.0"
    DATA_TYPE = ma.fields.Str(
        default=_DATA_TYPE, validate=[ma.validate.Equal(_DATA_TYPE)]
    )

    mappings = ma.fields.List(ma.fields.Nested(ExpressionIdentityMappingEntrySchema))


class IdentityMappingSchema(OneOfSchema):
    type_field = "DATA_TYPE"
    type_schemas = {
        ExternalIdentityMappingSchema._DATA_TYPE: ExternalIdentityMappingSchema,
        ExpressionIdentityMappingSchema._DATA_TYPE: ExpressionIdentityMappingSchema,
    }

    def get_obj_type(self, obj):
        if hasattr(obj, "DATA_TYPE"):
            return obj.DATA_TYPE
        elif hasattr(obj, "__getitem__"):
            return obj["DATA_TYPE"]
        else:
            raise Exception("Unknown object type: {}".format(obj.__class__.__name__))

    type_field_remove = False


expression_identity_mapping_entry_schema = ExpressionIdentityMappingEntrySchema()
expression_identity_mapping_schema = ExpressionIdentityMappingSchema()
external_identity_mapping_schema = ExternalIdentityMappingSchema()
identity_mapping_schema = IdentityMappingSchema()
