import marshmallow as ma
from .data_type import DataTypeSchema


class MissingRoleEntrySchema(ma.Schema):
    role = ma.fields.String(
        required=True,
        validate=[
            ma.validate.OneOf(
                [
                    "owner",
                    "administrator",
                    "access_manager",
                    "activity_manager",
                    "activity_monitor",
                    "access_monitor",
                ]
            )
        ],
    )
    collection = ma.fields.UUID()


class MissingRequiredRoleSchema(ma.Schema):
    _DATA_TYPE = "missing_required_role#1.0.0"
    DATA_TYPE = ma.fields.String(
        required=True,
        allow_none=False,
        default=_DATA_TYPE,
        validate=[ma.validate.Equal(_DATA_TYPE)],
    )
    roles = ma.fields.List(ma.fields.Nested(MissingRoleEntrySchema))


missing_required_role_schema = MissingRequiredRoleSchema()
DataTypeSchema.register_schema(missing_required_role_schema)
