from typing import List

import marshmallow as ma

from ..api.collection import CollectionSchema
from ..api.data_type import DataTypeSchema
from ..api.user_credential import UserCredentialSchema
from ..api.storage_gateway import StorageGatewaySchema


CEPH_CONNECTOR_ID = "1b6374b0-f6a4-4cf7-a26f-f262d9c6ca72"


class CephStoragePoliciesSchema(ma.Schema):
    _DATA_TYPE = "ceph_storage_policies#1.0.0"
    DATA_TYPE = ma.fields.Str(
        required=True,
        allow_none=False,
        default=_DATA_TYPE,
        validate=[ma.validate.Equal(_DATA_TYPE)],
    )
    s3_endpoint = ma.fields.Url()
    s3_buckets = ma.fields.List(ma.fields.Str)
    ceph_admin_key_id = ma.fields.Str()
    ceph_admin_secret_key = ma.fields.Str()

    private = ["ceph_admin_key_id", "ceph_admin_secret_key"]
    connector_id = CEPH_CONNECTOR_ID


class CephCollectionPoliciesSchema(ma.Schema):
    _DATA_TYPE = "ceph_collection_policies#1.0.0"
    DATA_TYPE = ma.fields.String(
        required=True,
        allow_none=False,
        default=_DATA_TYPE,
        validate=[ma.validate.Equal(_DATA_TYPE)],
    )
    private = []  # type: List[str]
    connector_id = CEPH_CONNECTOR_ID


class CephUserCredentialPoliciesSchema(ma.Schema):
    _DATA_TYPE = "ceph_user_credential_policies#1.0.0"
    DATA_TYPE = ma.fields.Str(
        required=True,
        allow_none=False,
        default=_DATA_TYPE,
        validate=[ma.validate.Equal(_DATA_TYPE)],
    )

    private = []  # type: List[str]
    connector_id = CEPH_CONNECTOR_ID


ceph_storage_policies_private_schema = CephStoragePoliciesSchema()
ceph_storage_policies_schema = CephStoragePoliciesSchema(
    load_only=CephStoragePoliciesSchema.private
)

ceph_user_credential_policies_schema = CephUserCredentialPoliciesSchema(
    load_only=CephUserCredentialPoliciesSchema.private
)
ceph_user_credential_policies_private_schema = CephUserCredentialPoliciesSchema()
ceph_collection_policies_schema = CephCollectionPoliciesSchema()

DataTypeSchema.register_schema(ceph_storage_policies_schema)
DataTypeSchema.register_schema(ceph_user_credential_policies_schema)
DataTypeSchema.register_schema(ceph_collection_policies_schema)

StorageGatewaySchema.register_schema(
    public=ceph_storage_policies_schema, private=ceph_storage_policies_private_schema
)

UserCredentialSchema.register_schema(
    public=ceph_user_credential_policies_schema,
    private=ceph_user_credential_policies_private_schema,
)

CollectionSchema.register_schema(ceph_collection_policies_schema)
