import marshmallow as ma

from ..cleanup_none_mixin import CleanupNoneMixin
from ..api.collection import CollectionSchema
from ..api.data_type import DataTypeSchema
from ..api.storage_gateway import StorageGatewaySchema
from ..api.user_credential import UserCredentialSchema


POSIX_CONNECTOR_ID = "145812c8-decc-41f1-83cf-bb2a85a2a70b"


class PosixStoragePoliciesSchema(ma.Schema, CleanupNoneMixin):
    _DATA_TYPE = "posix_storage_policies#1.0.0"
    DATA_TYPE = ma.fields.Str(
        required=True,
        allow_none=False,
        default=_DATA_TYPE,
        validate=[ma.validate.Equal(_DATA_TYPE)],
    )
    groups_allow = ma.fields.List(ma.fields.Str, allow_none=True)
    groups_deny = ma.fields.List(ma.fields.Str, allow_none=True)

    private = ["users_allow", "users_deny", "groups_allow", "groups_deny"]
    connector_id = POSIX_CONNECTOR_ID


class PosixCollectionPoliciesSchema(ma.Schema):
    _DATA_TYPE = "posix_collection_policies#1.0.0"
    DATA_TYPE = ma.fields.String(
        required=True,
        allow_none=False,
        default=_DATA_TYPE,
        validate=[ma.validate.Equal(_DATA_TYPE)],
    )
    connector_id = POSIX_CONNECTOR_ID


class PosixUserCredentialPoliciesSchema(ma.Schema):
    _DATA_TYPE = "posix_user_credential_policies#1.0.0"
    DATA_TYPE = ma.fields.Str(
        required=True,
        allow_none=False,
        default=_DATA_TYPE,
        validate=[ma.validate.Equal(_DATA_TYPE)],
    )
    connector_id = POSIX_CONNECTOR_ID


posix_storage_policies_schema = PosixStoragePoliciesSchema(
    load_only=PosixStoragePoliciesSchema.private
)
posix_storage_policies_private_schema = PosixStoragePoliciesSchema()
posix_user_credential_policies_schema = PosixUserCredentialPoliciesSchema()
posix_collection_policies_schema = PosixCollectionPoliciesSchema()

DataTypeSchema.register_schema(posix_storage_policies_schema)
DataTypeSchema.register_schema(posix_user_credential_policies_schema)
DataTypeSchema.register_schema(posix_collection_policies_schema)

StorageGatewaySchema.register_schema(
    public=posix_storage_policies_schema, private=posix_storage_policies_private_schema
)

UserCredentialSchema.register_schema(public=posix_user_credential_policies_schema)

CollectionSchema.register_schema(posix_collection_policies_schema)
