import marshmallow as ma


def create_patch_schema(
    schema_class, remove_required=None, remove_default=None, **kwargs
):
    """
    Instantiate a schema class for use with PATCH operations. Adds the
    following behaviors:

    -   Sets partial=True at the schema so that missing fields are allowed
    -   Sets the allow_none property of each field to True
    -   If remove_required (list of string or single string) is set, set the
        required property of named field(s) to False
    -   If remove_default (list of string or single string) is set, set the
        default property of named field(s) to ma.missing
    """
    if "partial" in kwargs:
        del kwargs["partial"]

    new_schema = schema_class(partial=True, **kwargs)
    for f in new_schema.fields:
        new_schema.fields[f].allow_none = True

    if remove_required is None:
        remove_required = []
    if remove_default is None:
        remove_default = []

    if isinstance(remove_required, str):
        remove_required = [remove_required]
    if isinstance(remove_default, str):
        remove_default = [remove_default]

    for p in remove_required:
        new_schema.fields[p].required = False
    for p in remove_default:
        new_schema.fields[p].default = ma.missing

    return new_schema
