import os

from cachelib import SimpleCache

import globus_sdk
from globus_sdk import AccessTokenAuthorizer
from globus_sdk.base import BaseClient

from globus.manager.api.utils.log import log_groups_exception


cache = SimpleCache(default_timeout=30)


class GroupsClient(BaseClient):
    default_headers = {"Content-Type": "application/json"}
    allowed_authorizer_types = [AccessTokenAuthorizer]

    def __init__(self, authorizer=None, **kwargs):
        if "base_url" not in kwargs:
            base_url = "https://groups.api.globus.org"
            environment = kwargs.get("environment")
            if not environment:
                environment = os.environ.get("GLOBUS_SDK_ENVIRONMENT")
            if environment:
                environment = environment.lower()
                if environment in ["staging", "test", "integration", "sandbox"]:
                    base_url = "https://groups.api.{}.globuscs.info".format(environment)
                elif environment == "preview":
                    base_url = "https://groups.api.preview.globus.org"
                elif environment in ["production", "default"]:
                    base_url = "https://groups.api.globus.org"
                else:
                    raise ValueError("Unknown environment")
            kwargs["base_url"] = base_url
        super().__init__("groups", base_path="/v2", authorizer=authorizer, **kwargs)

    def get_my_groups(self, force_recheck=False):
        cache_key = "my_groups_response_{}".format(self.authorizer.access_token)

        if force_recheck:
            cache.delete(cache_key)
            groups_response = None
        else:
            groups_response = cache.get(cache_key)

        if groups_response is None:
            try:
                groups_response = self.get("/groups/my_groups")
            except globus_sdk.exc.GlobusAPIError as e:
                log_groups_exception(e)
                raise

            cache.set(cache_key, groups_response)

        return groups_response
