import time

from cachelib import SimpleCache
from flask import current_app
import globus_sdk

import globus.manager.api.errors as errors


cache = SimpleCache()


def get_dependent_token(
    token, resource_server="transfer.api.globus.org", force_new_token=False
):
    cache_key_fmt = "dependent_token_response_{}_{{}}".format(token)
    desired_cache_key = cache_key_fmt.format(resource_server)
    dependent_token = None
    execution_count = 1

    if force_new_token:
        cache.delete(desired_cache_key)
        dependent_token = None
    else:
        dependent_token = cache.get(desired_cache_key)

    dependent_token_exp = 0
    while execution_count < 10 and time.time() > dependent_token_exp - 120:
        execution_count += 1
        if dependent_token is None:
            try:
                token_response = current_app.auth_client.oauth2_get_dependent_tokens(
                    token
                ).by_resource_server
                current_app.logger.debug(
                    "token_response for token=%s resource_server=%s is %s",
                    token,
                    resource_server,
                    token_response,
                )
                for r in token_response:
                    cache_key = cache_key_fmt.format(r)
                    token_exp = token_response[r]["expires_at_seconds"]
                    cache_expiration_seconds = int((token_exp - time.time()) / 2)
                    cache.set(
                        cache_key, token_response[r], timeout=cache_expiration_seconds
                    )
                    if r == resource_server:
                        dependent_token = token_response[r]
                        dependent_token_exp = token_exp
            except globus_sdk.GlobusAPIError as e:
                current_app.logger.warning(
                    "Error performing dependent token grant: %s", str(e)
                )
        else:
            dependent_token_exp = dependent_token["expires_at_seconds"]

        if time.time() > dependent_token_exp - 120:
            cache.delete(cache_key)
            dependent_token = None
            dependent_token_exp = 0

    if dependent_token is None:
        raise errors.InternalServerError("Unable to get valid dependent token")

    return dependent_token["access_token"]
