import click

from globus.manager.version import __version__ as package_version

from gcs_cli.version import __version__ as cli_version
from gcs_cli.commands import main as gcs_cli_main
from gcs_cli.commands.endpoint import endpoint_command
from gcs_cli.commands.node import node_command

from .endpoint_setup import endpoint_setup
from .endpoint_cleanup import endpoint_cleanup
from .node_setup import node_setup
from .node_cleanup import node_cleanup
from .node_set_configuration import node_set_configuration
from .node_update_vhost import node_update_vhost
from .assistant.assistant import assistant_command
from .migrate53.migrate53 import migrate53_command
from .reaper import reap


version_message = "globus-connect-server, package {}, cli {}".format(
    package_version, cli_version
)


def update_version(command_or_group):
    result = click.version_option(
        package_version, message=version_message, hidden=True
    )(command_or_group)

    if isinstance(command_or_group, click.Group):
        for c in command_or_group.commands:
            result.commands[c] = update_version(command_or_group.commands[c])
    return result


endpoint_command.add_command(endpoint_setup)
endpoint_command.add_command(endpoint_cleanup)
endpoint_command.add_command(migrate53_command)
node_command.add_command(node_setup)
node_command.add_command(node_cleanup)
node_command.add_command(node_set_configuration)
node_command.add_command(node_update_vhost)
gcs_cli_main.add_command(assistant_command)
gcs_cli_main.add_command(reap)

main = update_version(gcs_cli_main)


if __name__ == "__main__":
    main()
